import QtQuick 1.0
import QtMultimediaKit 1.1
import "scripts/dateandtime.js" as DT

Page {
    id: root

    property bool videoPlaying: true
    property variant currentVideo: []
    property int playlistPosition: 0
    property bool gettingVideoUrl: false

    function setPlaylist(videoList) {
        playbackQueue.clear();
        for (var i = 0; i < videoList.length; i++) {
            playbackQueue.append(videoList[i]);
        }
        playlistPosition = 0;
        currentVideo = playbackQueue.get(0);
        startPlayback();
    }

    function appendPlaylist(videoList) {
        if (playbackQueue.count == 0) {
            playlistPosition = 0;
            currentVideo = videoList[0];
        }
        for (var i = 0; i < videoList.length; i++) {
            playbackQueue.append(videoList[i]);
        }
    }

    function startPlayback() {
        if (currentVideo.archive) {
            videoPlayer.setVideo(currentVideo.filePath);
        }
        else {
            gettingVideoUrl = true;
            if (currentVideo.youtube) {
                YouTube.getVideoUrl(currentVideo.videoId);
            }
            else {
                Dailymotion.getVideoUrl(currentVideo.id);
            }
        }
    }

    function previous() {
        /* Play the previous item in the playlist */

        if (playlistPosition > 0) {
            playlistPosition--;
            getNextVideo();
        }
    }

    function next() {
        /* Play the next item in the playlist */

        if (playlistPosition < playbackQueue.count - 1) {
            playlistPosition++;
            getNextVideo();
        }
        else {
            stopPlayback();
        }
    }

    function getNextVideo() {
        if (playlistPosition < playbackQueue.count) {
            currentVideo = playbackQueue.get(playlistPosition);
            if (currentVideo.archive) {
                videoPlayer.setVideo(currentVideo.filePath);
            }
            else if (currentVideo.videoUrl) {
                videoPlayer.setVideo(currentVideo.videoUrl);
            }
            else {
                gettingVideoUrl = true;
                YouTube.getVideoUrl(currentVideo.videoId);
            }
        }
    }

    function stopPlayback() {
        videoPlayer.stop();
        videoPlayer.source = "";
        currentVideo = [];
        pageStack.pop();
        MainWindow.setScreenOrientation(Settings.screenOrientation);
        Utils.doNotDisturb(false);
    }

    function rateVideo(rating) {
        if (!(currentVideo.rating) && (YouTube.userSignedIn)) {
            likeOrDislike = rating;
            YouTube.rateVideo(currentVideo.videoId, likeOrDislike);
        }
    }

    onCurrentVideoChanged: {
        if ((currentVideo.videoId) && (transfers.videoInDownloads(currentVideo.videoId))) {
            var video = currentVideo;
            video["download"] = true;
            currentVideo = video;
        }
    }

    Component.onCompleted: {
        MainWindow.setScreenOrientation("landscape");
        Utils.doNotDisturb(true);
    }

    Connections {
        target: YouTube
        onGotVideoUrl: {
            gettingVideoUrl = false;
            videoPlayer.setVideo(videoUrl);
            var cv = currentVideo;
            cv["videoUrl"] = videoUrl;
            currentVideo = cv;
        }
        onVideoUrlError: {
            gettingVideoUrl = false;
            next();
        }
        onVideoRated: {
            if ((currentVideo.videoId) && (id == currentVideo.videoId)) {
                var cv = currentVideo;
                cv["rating"] = likeOrDislike;
                currentVideo = cv;
                messages.displayMessage(messages._VIDEO_RATED);
            }
        }
        onAddedToFavourites: {
            if ((currentVideo.videoId) && (id == currentVideo.videoId)) {
                var cv = currentVideo;
                cv["favourite"] = true;
                currentVideo = cv;
            }
        }
    }

    Connections {
        target: Dailymotion
        onGotVideoUrl: {
            gettingVideoUrl = false;
            videoPlayer.setVideo(videoUrl);
            var cv = currentVideo;
            cv["videoUrl"] = videoUrl;
            currentVideo = cv;
        }
        onVideoUrlError: {
            gettingVideoUrl = false;
            next();
        }
        onAddedToFavourites: {
            if (pageStack.currentPage == root) {
                var cv = currentVideo;
                cv["favourite"] = true;
                currentVideo = cv;
            }
        }
    }

    Connections {
        target: transfers
        onDownloadAdded: {
            if ((currentVideo.videoId) && (currentVideo.videoId == videoId)) {
                var video = currentVideo;
                video["download"] = true;
                currentVideo = video;
            }
        }
    }

    ContextMenu {
        id: contextMenu

        titleText: qsTr("Options")
        menuItems: [
            ContextMenuItem {
                text: qsTr("Toggle aspect ratio")
                closeMenuWhenClicked: false
                onClicked: videoPlayer.fillMode == Video.Stretch ? videoPlayer.fillMode = Video.PreserveAspectFit : videoPlayer.fillMode == Video.PreserveAspectFit ? videoPlayer.fillMode = Video.PreserveAspectCrop : videoPlayer.fillMode = Video.Stretch
            },

            ContextMenuItem {
                text: qsTr("Repeat")
                checked: videoPlayer.repeat
                closeMenuWhenClicked: false
                onClicked: videoPlayer.repeat = !videoPlayer.repeat
            }
        ]
    }

    SelectionDialog {
        id: queueDialog

        titleText: qsTr("Playback queue")
        model: ListModel {
            id: playbackQueue
        }
        destroyOnClosing: false
        emitValueChosenSignal: false
        onAccepted: {
            playlistPosition = selectedIndex;
            getNextVideo();
        }
        onStateChanged: if (state == "open") selectedIndex = playlistPosition;
    }

    ToolBar {
        id: toolBar

        property bool show: false

        z: 10
        anchors { left: parent.left; right: parent.right; top: parent.bottom }

        states: State {
            name: "show"
            when: toolBar.show
            AnchorChanges { target: toolBar; anchors { top: undefined; bottom: parent.bottom } }
        }

        transitions: Transition {
            AnchorAnimation { easing.type: Easing.OutQuart; duration: 300 }
        }

        tools: Item {
            anchors.fill: parent

            ToolIcon {
                id: stopButton

                anchors { left: parent.left; verticalCenter: parent.verticalCenter }
                iconSource: Settings.appTheme == "light" ? "images/back.png" : "images/back-white.png"
                onClicked: stopPlayback()
            }

            ToolIcon {
                id: playButton

                anchors { left: stopButton.right; leftMargin: 10; verticalCenter: parent.verticalCenter }
                iconSource: videoPlayer.paused ? Settings.appTheme == "light" ? "images/play.png" : "images/play-white.png" : Settings.appTheme == "light" ? "images/pause.png" : "images/pause-white.png"
                onClicked: videoPlayer.paused = !videoPlayer.paused
            }

            ProgressBar {
                id: progressBar

                width: 450
                anchors.centerIn: parent
                value: toolBar.show ? videoPlayer.position / videoPlayer.duration : 0
            }

            Text {
                anchors { top: progressBar.bottom; horizontalCenter: progressBar.left }
                font.pixelSize: _SMALL_FONT_SIZE
                color: _TEXT_COLOR
                text: (!toolBar.show) || (videoPlayer.status == Video.Loading) || (videoPlayer.status == Video.Buffering) ? "0:00" : DT.getTime(videoPlayer.position)
            }

            Text {
                anchors { top: progressBar.bottom; horizontalCenter: progressBar.right }
                font.pixelSize: _SMALL_FONT_SIZE
                color: _TEXT_COLOR
                text: toolBar.show ? DT.getTime(videoPlayer.duration) : "0:00"
            }

            SeekBubble {
                id: seekBubble

                z: 1000
                anchors.bottom: progressBar.top
                opacity: value != "" ? 1 : 0
                value: (seekMouseArea.drag.active) && (seekMouseArea.posInsideDragArea) ? DT.getTime(Math.floor((seekMouseArea.mouseX / seekMouseArea.width) * videoPlayer.duration)) : ""
            }

            MouseArea {
                id: seekMouseArea

                property bool posInsideMouseArea: false
                property bool posInsideDragArea: (seekMouseArea.mouseX >= 0) && (seekMouseArea.mouseX <= seekMouseArea.drag.maximumX - 125)

                width: progressBar.width
                height: 60
                anchors.centerIn: progressBar
                drag.target: seekBubble
                drag.axis: Drag.XAxis
                drag.minimumX: -40
                drag.maximumX: width + 125
                enabled: !currentVideo.dailymotion ? true : false
                onExited: posInsideMouseArea = false
                onEntered: posInsideMouseArea = true
                onPressed: {
                    posInsideMouseArea = true;
                    seekBubble.x = mouseX + 125
                }
                onReleased: if (posInsideMouseArea) videoPlayer.position = Math.floor((mouseX / width) * videoPlayer.duration);
            }

            ToolIcon {
                id: queueButton

                anchors { right: menuButton.left; rightMargin: 10; verticalCenter: parent.verticalCenter }
                iconSource: Settings.appTheme == "light" ? "images/list.png" : "images/list-white.png"
                onClicked: queueDialog.open()
            }

            ToolIcon {
                id: menuButton

                anchors { right: parent.right; verticalCenter: parent.verticalCenter }
                iconSource: Settings.appTheme == "light" ? "images/menu.png" : "images/menu-white.png"
                onClicked: contextMenu.open()
            }
        }
    }

    Timer {
        id: controlsTimer

        running: (toolBar.show) && (!seekMouseArea.pressed)
        interval: 3000
        onTriggered: toolBar.show = false
    }

    Video {
        id: videoPlayer

        property bool repeat: false // True if playback of the current video is to be repeated

        function setVideo(videoUrl) {
            videoPlayer.source = videoUrl;
            videoPlayer.play();
        }

        width: 800
        height: 480
        fillMode: Video.PreserveAspectFit
        anchors.centerIn: parent
        opacity: position > 0 ? 1 : 0
        onError: {
            if ((error != Video.NoError) && (videoPlayer.source != "")) {
                if (error == Video.NetworkError) {
                    messages.displayMessage(messages._NETWORK_ERROR);
                }
                else if (error == Video.FormatError) {
                    messages.displayMessage(messages._FORMAT_ERROR);
                }
                else if (error == Video.ResourceError) {
                    messages.displayMessage(messages._PLAYBACK_ERROR);
                }
                next();
            }
        }
        onStatusChanged: {
            if (videoPlayer.status == Video.EndOfMedia) {
                if (videoPlayer.repeat) {
                    videoPlayer.position = 0;
                    videoPlayer.play();
                }
                else {
                    next();
                }
            }
        }
        onPositionChanged: if ((videoPlayer.position > 2000) && ((videoPlayer.duration - videoPlayer.position) < 500) && (playlistPosition == playbackQueue.count - 1) && (!videoPlayer.repeat)) stopPlayback(); // waiting for status == Video.EndOfMedia can cause problems if it's the last video in the playlist
    }

    MouseArea {
        id: videoMouseArea

        property int xPos

        anchors.fill: videoPlayer
        onPressed: xPos = mouseX
        onReleased: {
            if (videoPlayer.paused) {
                videoPlayer.paused = false;
            }
            else if (((xPos - mouseX) > 100) && (playlistPosition < playbackQueue.count - 1)) {
                next();
            }
            else if ((mouseX - xPos) > 100) {
                previous();
            }
            else {
                toolBar.show = !toolBar.show;
            }
        }
    }

    Text {
        anchors.centerIn: videoPlayer
        font.pixelSize: _LARGE_FONT_SIZE
        color: "white"
        text: qsTr("Loading...")
        visible: ((videoPlayer.status == Video.Loading) || (videoPlayer.status == Video.Buffering) || (gettingVideoUrl)) && (!videoPlayer.paused)
    }

    Image {
        anchors.centerIn: videoPlayer
        source: videoMouseArea.pressed ? "images/play-button-" + Settings.activeColorString + ".png" : "images/play-button.png"
        visible: videoPlayer.paused
    }
}
